<?php 	/*
	Plugin Name: Glassboxx for WooCommerce
	Plugin URI: https://glassboxx.com
	Description: Adds Glassboxx integration to your Woocommerce store so fulfilment of sales of virtual products for audiobooks and eBooks uses Glassboxx with full DRM using the Glassboxx apps. To set up your integration, sign up for a free vendor account at https://glassboxx.com/signup, upload your assets and meta data, and enter the Glassboxx API credentials from your account into the configuration page at Woocommerce/Settings/Advanced/Glassboxx Integration.
	Version: 2.0.7
	Author: Glassboxx
	Author URI: https://glassboxx.com/
	Text Domain: firsty
	Domain Path: /languages
	License: GPLv3
	WC requires at least: 3.0.0
	WC tested up to: 6.3.1
	Woo: 123456:abc123def456ghi789jkl012mno345pq
	WooCommerce: true
*/

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

// Declare compatibility with WooCommerce features
add_action( 'before_woocommerce_init', function() {
    if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'remote_logging', __FILE__, true );
    }
});

class Woo_Glassboxx {

	function __construct() {

		/**
		 ** Check woocommerce is active
		**/
		$all_plugins = apply_filters('active_plugins', get_option('active_plugins'));
		if (stripos(implode($all_plugins), 'woocommerce.php')) {
			define('GBXURL', plugin_dir_url( __FILE__ ));
			define('GBXSERVER', 'https://server.glassboxx.co.uk/');
			if( is_admin() ) {

				/**
				 **  load admin settings  
				**/
				add_filter('woocommerce_get_sections_advanced', array( $this, 'glassboxx_admin' ));
				add_filter('woocommerce_get_settings_advanced', array( $this, 'glassboxx_settings'), 10, 2);
        		add_action('woocommerce_settings_save_advanced', array( $this, 'glassboxx_admin_auth'), 99);
				add_action('admin_enqueue_scripts', array( $this, 'glassboxx_admin_css' ));
				
				/**
				 ** sends orders to glassboxx if admin user changes order status from 'Pending Payment' to 'Processing' or 'Completed'
				**/
				add_action( 'woocommerce_order_status_processing', array( $this, 'glassboxx_control'), 10, 1);
				add_action( 'woocommerce_order_status_completed', array( $this, 'glassboxx_control'), 10, 1);
				
				
			} else {

				/**
				 ** apply actions to the payment process
				**/	
			    add_action('woocommerce_payment_complete', array( $this, 'glassboxx_control'), 50, 1);
				add_filter('woocommerce_thankyou_order_received_text', array($this, 'glassboxx_order_success_message'), 10, 1);
				add_action('woocommerce_email_before_order_table', array($this, 'glassboxx_email_order_complete'), 10, 4 );
				
				
				/**
				 **  apply action to url to find products ../findproduct?sku=xxxxxx
				**/
				add_action('init', array($this, 'add_glassboxx_endpoint'));
				add_action('template_redirect', array($this, 'find_the_glassboxx_product'), 1, 1);

				
				/**
				 **	apply result to hook depending on WooCommerce version
				**/
				$result_action = 'woocommerce_before_thankyou';

				if(!$this->woo_version_checker()) {
					// WooCommerce v4.1 & below
					$result_action = 'woocommerce_thankyou';
				}
				add_action($result_action, array($this, 'glassboxx_result'), 9, 1);
			}
		}
	}

	function woo_version_checker(){
		$version = get_option( 'woocommerce_version' ); // WC()->version;
		if($version) {
			if (version_compare( $version, '4.2', ">=" )) {
				return true;
			}
			return false;
		}
	}

	function add_glassboxx_endpoint() {
		add_query_arg('sku', 0);
		add_rewrite_endpoint('findproduct', EP_ROOT, true);
	}


	function find_the_glassboxx_product($template){
		global $wp_query, $woocommerce;
		if('findproduct' === $wp_query->query_vars['name']) {
			$location = home_url();
			$id = wc_get_product_id_by_sku( $_GET['sku'] );
			if($id) {
				$product = wc_get_product($id);
				if($product->get_type() === 'grouped') {
					$products = $product->get_children();
					$location = get_permalink($id);
					foreach($products as $p) {
						$child = wc_get_product($p);
						if($child->get_virtual()) {
							$location = get_permalink($p);
							break;
						}
					}
					if($location === get_permalink($id)) {
						foreach($products as $p) {
							$child = wc_get_product($p);
							if($child->get_downloadable()) {
								$location = get_permalink($p);
								break;
							}
						}					
					}
				} else {
					$location = get_permalink($id);	
				}
			}
			wp_redirect($location);
			die;			

		} else {
			return $template;
		}
	}

	function glassboxx_server($stage){
        $urls = array(
        'auth'  => "rest/V1/integration/admin/token",
        'cust'  => "rest/V1/glassboxxorder/customCustomer",
        'order'  => "rest/V1/glassboxxorder/toglassboxx",
		);
        return GBXSERVER . $urls[$stage];
	}

  function glassboxx_auth($opts){
    $token = false;
    $query = array(
      'username' => $opts['glassboxx_wc_user'],
      'password' => $opts['glassboxx_wc_apikey'],
    );
    $headers = array(
      'Content-Type' => 'application/json; charset=utf-8',
    );

    $response = wp_safe_remote_post($this->glassboxx_server('auth'), array(
      'method'      => 'POST',
      'headers'     => $headers,
      'body'        => json_encode($query),
      'data_format' => 'body',
    ));
    
    if(!is_wp_error($response) && $response['response']['code'] === 200) {
      $token = sprintf('Bearer %s', json_decode($response['body']));
	}

    return $token;
  }

  function glassboxx_admin_auth(){
	$opts = $this->gbx_ready();
    if($opts) {
      $authenticate = $this->glassboxx_auth($opts);
      if($authenticate !== false) {
		WC_Admin_Settings::add_message('Glassboxx API details successfully verified');
      } else {
		WC_Admin_Settings::add_error('Glassboxx API details failed verification, please check and try again. ');
	  }
    } else if(null !== get_option('glassboxx_wc_active') && 'no' !== get_option('glassboxx_wc_active')) {
      WC_Admin_Settings::add_error('Glassboxx integration disabled. Please fill in all the fields.');
    }
  }
	function glassboxx_control($id){
		if(get_post_meta( $id, 'gbx_sent', true ) == 1) {
			return;
		}
		$opts = $this->gbx_ready();
		if($opts) {
			$this->glassboxx_log('');
			$this->glassboxx_log('Ready to connect Glassboxx API...');
			$this->glassboxx_log('Logging order...');
			$this->glassboxx_log('ORDER ID: '.$id);
			$items = $this->gbx_order_items($id);
			if($items) {
				$auth = $this->glassboxx_auth($opts);
				if($auth !== false){
					
					$this->glassboxx_log('Glassboxx API details successfully verified');
					
					$headers = array(
						'Content-Type' => 'application/json; charset=utf-8',
						'Authorization' => $auth,
					);
					$customer_query = array( 
						'customer' => array(
							'email' 		=> $items['customer']['email'],
							'firstname' 	=> $items['customer']['fname'],
							'lastname'		=> $items['customer']['lname'],
							'created_at' 	=> $items['customer']['date'],
							'created_in'	=> $items['customer']['cin'],
						)
					);
					
					unset($items['customer']);
					$customer = wp_safe_remote_post($this->glassboxx_server('cust'), array(
						'method'			=> 'POST',
						'headers' 		=> $headers,
						'body'				=> json_encode($customer_query),
						'data_format'	=> 'body',
					));
					
					$this->glassboxx_log("Customer json query: ".json_encode($customer_query));
					
					if(!is_wp_error($customer) && $customer['response']['code'] === 200 && json_decode($customer['body']) === 'Success'){
						
						$this->glassboxx_log("Customer json request sent to glassboxx api: SUCCESS");
						
						$order_query = ['items' => []];
						foreach($items as $item) {
							$order_query['items'][] = $item;
						}
						$orders = wp_safe_remote_post($this->glassboxx_server('order'), array(
							'method'			=> 'POST',
							'headers'			=> $headers,
							'body'				=> json_encode($order_query),
							'data_format' => 'body',
						));
						
						$this->glassboxx_log("Order item(s) json query: ".json_encode($order_query));
						
						if(!is_wp_error($orders) && $orders['response']['code'] === 200){
							$result = array(true, 'Success');
							$this->glassboxx_log('Order item(s) request sent to glassboxx api: SUCCESS');
							update_post_meta( $id, 'gbx_sent', 1 );
						} else {
							$result = array(false, 'Failed to add the order to Glassboxx');
							$this->glassboxx_log('Order item(s) json request failed!');
						}
					} else {
						$result = array(false, 'Failed to add customer to Glassboxx');
						$this->glassboxx_log('Customer json request failed!');
					}			
				} else {
					$result = array(false, 'Glassboxx integration failed!');
					$this->glassboxx_log('Glassboxx integration failed!');
				}	
				if( !is_admin() ){
					WC()->session->set('wc_gbox_result', $result);
				}
			}
			else
			{
				$this->glassboxx_log('Order does not contain any glassboxx product');
			}
		}
		else
		{
			$this->glassboxx_log('Glassboxx plugin not activated ');
		}
	}
	
	function glassboxx_order_success_message($str){
		$v = get_option('glassboxx_ty_msg');
		$v = empty($v) ? $str : $v;
		return $v;
	}

	function glassboxx_email_order_complete($order, $sent_to_admin, $plain_text, $email){
		if ( $email->id == 'customer_processing_order' || $email->id == 'customer_completed_order' ) {
			$v = get_option('glassboxx_ty_msg');
			if(!empty($v)) {
				echo wp_kses_post( wpautop($v));
			}
		}
	}

	function glassboxx_result($id){
		wp_enqueue_style('gbx-css', GBXURL . "/css/thankyou.css");
		$result = WC()->session->get('wc_gbox_result');
		if(is_array($result)) {
			WC()->session->set( 'wc_gbox_result' , null ); 
			$items = $this->gbx_order_items($id);
			if($items) {
				?><div class="container"><?php
				if($result[0] === true && empty(get_option('glassboxx_ty_msg'))) {
					echo wp_kses_post($this->glassboxx_thankyou_html()); ?>
				<?php } else if($result[0] !== true) { ?>
					<p>Sorry, it looks like there's been a problem: <?php $result[1]; ?>  Please contact the site administrator.</p>
				<?php } ?>
			</div><?php 
			}
		}
	}

	function gbx_order_items($order_id){
		$virtual = [];
		$order = wc_get_order($order_id);
		foreach( $order->get_items() as $item_id => $item) {
			if(! is_object($item) ) {
				continue;
			}
			if($item->is_type('line_item') ) {
				$product = $item->get_product();
                if ($product->is_virtual('yes')) {
                    $discount = 0;
                    //This is the discount for a catalog discount
                    if ($product->get_sale_price() != '') {
                        $discount = $product->get_regular_price() - $product->get_sale_price();
                    	//This is a discount for a coupon discount. This is inside here in case a customer will apply a coupon discount
                        //on top of the catalog discount
                        if ($item->get_subtotal() != '') {
                            $discount += $item->get_subtotal() - $item->get_total();
                        }

                    } elseif ($item->get_subtotal() != '') {
                        $discount = $item->get_subtotal() - $item->get_total();
                    }
	
					if($order->get_date_paid())
					{
						$paid_date = $order->get_date_paid()->format('Y-m-d H:i:s');
					}
					else
					{
						$paid_date = date('Y-m-d H:i:s');
					}
					
                    $virtual[] = array(
                        'currency_code' => $order->get_currency(),
                        'sku' => $product->get_sku(),
                        'created_at' => $paid_date,
                        'discount_amount' => $discount,
                        'original_order_number' => $order->get_id(),
                        'customer_email' => $order->get_billing_email(),
                        'hostname' => get_option('glassboxx_wc_id'),
                        'price_incl_tax' => $product->get_regular_price(),
                        'type_of_interaction' => 'purchase',
                    );
                }
            }
        }
		if(empty($virtual)){
			$virtual = false;
		} else {
			$virtual['customer'] = array(
				'email' => $order->get_billing_email(),
				'fname' => $order->get_billing_first_name(),
				'lname' => $order->get_billing_last_name(),
				'date' 	=> $paid_date,
				'cin'		=> home_url(),
			);
		}
		return $virtual;
	}

	private function glassboxx_options(){
		return array(
			'glassboxx_wc_active',
			'glassboxx_wc_id',
			'glassboxx_wc_user',
			'glassboxx_wc_apikey',
		);
	}

	function gbx_ready(){
		$opts = $this->glassboxx_options();
		$vals = array();
		foreach($opts as $o) {
			$v = get_option($o);
			if($v === 'no') {
				$v = false;
			}
			$vals[$o] = $v;
		}
		for($i=0; $i < count($vals); $i++) {
			if(in_array(false,$vals)) {
				$vals = false;
				break;
			}
		}
		return $vals;
	}

	function glassboxx_admin($sections) {
		$sections['gbx_woo'] = 'Glassboxx Integration';
		return $sections;
	}

	function glassboxx_settings( $settings, $section ){
		if('gbx_woo' === $section) {
			$settings = array(
				'name' => 'Glassboxx Integration',
				array(
					'title' => 'Glassboxx Configuration',
					'type' => 'title',
					'id'	=> 'glassboxx_wc',
					'desc'	=> 'To use Glassboxx for audiobook and ebook fulfilment, signup for a free account at <a href="https://glassboxx.com/signup" target="_blank" rel="noopener noreferrer">glassboxx.com/signup</a> and upload your assets and meta data. Then to integrate this site with your Glassboxx account, log in at <a href="https://glassboxx.com/login" target="_blank" rel="noopener noreferrer">glassboxx.com/login</a> and copy your vendor id, token username and token password from the INTEGRATIONS/Vendor page and paste them below.<br><br>In your WooCommerce store, your audiobooks and/or eBooks should be set up as Simple Virtual products with SKUs that match the corresponding ISBNs uploaded to Glassboxx.<br><br>If you need any help, please contact us at <a href="mailto:glassboxxsupport@firstygroup.com">glassboxxsupport@firstygroup.com</a>. ',
				),
				array(
					'title' 	=> 'Enable Glassboxx Integration',
					'id' 		=> 'glassboxx_wc_active',
					'type'		=> 'checkbox',
					'default'	=> 'no',
				),
				array(
					'title' 	=> 'Vendor ID',
					'id' 		=> 'glassboxx_wc_id',
					'type'		=> 'text',
					'desc_tip' 	=> 'You will find this on the INTEGRATIONS/Vendor page in your Glassboxx vendor area',
				),
				array(
					'title' 	=> 'Token Username',
					'id' 		=> 'glassboxx_wc_user',
					'type'		=> 'text',
					'desc_tip' 	=> 'You will find this on the INTEGRATIONS/Vendor page in your Glassboxx vendor area',
				),
				array(
					'title' 	=> 'Token Password',
					'id' 		=> 'glassboxx_wc_apikey',
					'type'		=> 'password',
					'desc_tip' 	=> 'You will find this on the INTEGRATIONS/Vendor page in your Glassboxx vendor area',
					'desc'      => '<button type="button" class="show-hide-password" tabindex="-1"><span class="dashicons dashicons-visibility"></span></button>'
				),
				array(
					'title'		=> 'Order Completion Message',
					'id'		=> 'glassboxx_ty_msg',
					'type'		=> 'textarea',
					'desc_tip'	=> 'Text displayed to end customer on the order success page when they successfully place an order',
					'default' 	=> $this->glassboxx_thankyou_html(),
					'desc' 		=> 'To revert to the default Glassboxx thank you message, copy/paste the following into the Order Completion Message field below<br><br>
									<textarea cols="600" readonly>'. $this->glassboxx_thankyou_html() .'</textarea><br><br><p>Order Completion Message</p>',
				),
				array(
					'title' 	=> 'Enable Debug Mode',
					'id' 		=> 'glassboxx_wc_debug',
					'type'		=> 'checkbox',
					'default'	=> 'no',
					'desc' 	    => 'If logging is enabled, a log file called woocommerce-glassboxx.log will be generated in the /wp-content/uploads directory',
				),
				array( 'type' => 'sectionend', 'id' => 'glassboxx_wc', ),
			);
		}
		return $settings;
	}

	function glassboxx_admin_css($page){
		if($page === 'woocommerce_page_wc-settings') {
			wp_enqueue_style('dashicons');
			wp_enqueue_style('glassboxx-css', GBXURL . '/css/admin.css', false, null);
			
			// Add custom CSS for password field styling
			$custom_css = '
				.password-input-wrapper {
					position: relative;
					max-width: 400px;
					display: inline-block;
				}
				.password-input-wrapper input[type="password"],
				.password-input-wrapper input[type="text"] {
					width: 100%;
					padding-right: 35px;
				}
				.show-hide-password {
					position: absolute;
					right: 5px;
					top: 50%;
					transform: translateY(-50%);
					background: none !important;
					border: none !important;
					padding: 0 !important;
					box-shadow: none !important;
					cursor: pointer;
					color: #666;
					width: 30px;
					height: 30px;
					display: flex;
					align-items: center;
					justify-content: center;
				}
				.show-hide-password:hover {
					color: #000;
				}
				.show-hide-password .dashicons {
					width: 20px;
					height: 20px;
					font-size: 20px;
					display: flex;
					align-items: center;
					justify-content: center;
				}
			';
			wp_add_inline_style('glassboxx-css', $custom_css);
			
			// Add inline script for password toggle
			$script = '
				jQuery(document).ready(function($) {
					// Wrap password field in a container
					var passwordField = $("#glassboxx_wc_apikey");
					if (!passwordField.parent().hasClass("password-input-wrapper")) {
						passwordField.wrap("<div class=\'password-input-wrapper\'></div>");
						$(".show-hide-password").insertAfter(passwordField);
					}

					$(".show-hide-password").on("click", function(e) {
						e.preventDefault();
						var passwordField = $("#glassboxx_wc_apikey");
						var icon = $(this).find(".dashicons");
						
						if (passwordField.attr("type") === "password") {
							passwordField.attr("type", "text");
							icon.removeClass("dashicons-visibility").addClass("dashicons-hidden");
						} else {
							passwordField.attr("type", "password");
							icon.removeClass("dashicons-hidden").addClass("dashicons-visibility");
						}
					});
				});
			';
			wp_add_inline_script('jquery', $script);
		}
	}


	function glassboxx_thankyou_html() { 

		$appstore_img       = plugin_dir_url( __FILE__ )."img/appstore-icon.png";
		$googleplay_img     = plugin_dir_url( __FILE__ )."img/googleplay-icon.png";
		$macos_img          = plugin_dir_url( __FILE__ )."img/glassboxx-macos.png";
		$windows_img        = plugin_dir_url( __FILE__ )."img/glassboxx-windowsstore.png";
		
		return '<h2 class="woocommerce-order-details__title">Thank you for your order</h2>
		<p><strong>If your order contains audiobooks and/or ebooks you should use the Glassboxx app on your iOS or Android phone or tablet, or Windows or Mac computers to listen or read your books.</strong></p>
		<ul>
			<li>If you already have <strong>Glassboxx</strong> on your phone or tablet, go to MY BOOKS to see your new purchases.</li>
			<li>If you don\'t yet have the <strong>Glassboxx</strong> app, click the relevant link below to install it, and log in with the email address used for your purchase. The audiobooks and/or eBooks you\'ve just bought will be displayed on MY BOOKS.</li>
			<li>Make sure you use the email address used for your purchase when you log in.</li>
			<li>It can take a few minutes for books to appear in the app. After logging in, tap the refresh option to make sure you have received your latest books.</li>
		</ul>				
		<div class="gbx-cols">
			<a href="https://apps.apple.com/gb/app/glassboxx/id1464705712" title="Click here for the Glassboxx iOS app on the Apple iOS App store"><img src='.$appstore_img.' alt="Glassboxx app on the Apple iOS App store" /></a>
		</div>
		<div class="gbx-cols">
			<a href="https://play.google.com/store/apps/details?id=uk.co.firstygroup.glassboxx" title="Click here for the Glassboxx Android app on the Google Play store"><img src='.$googleplay_img.'  alt="Glassboxx app on the Google Play store"/></a>
		</div>
		<div class="gbx-cols">
			<a href="https://apps.microsoft.com/store/detail/glassboxx/9NZKLR5V4FQ6" title="Click here for the Glassboxx Windows app on the Microsoft store"><img src='.$windows_img.'  alt="Glassboxx app on the Microsoft store"/></a>
		</div>
		<div class="gbx-cols">
			<a href="https://glassboxx.com/downloads/Glassboxx-1.3.29.dmg" title="Click here for the Glassboxx macOS app"><img src='.$macos_img.'  alt="Glassboxx macOS app download"/></a>
		</div>
		<p></p>';
	}
	
	
	function glassboxx_log( $entry, $mode = 'a', $file = 'woocommerce-glassboxx' ) { 
	
		if(get_option('glassboxx_wc_debug')=='yes'){
			// Get WordPress uploads directory.
			$upload_dir = wp_upload_dir();
			$upload_dir = $upload_dir['basedir'];

			// If the entry is array, json_encode.
			if ( is_array( $entry ) ) { 
			  $entry = json_encode( $entry ); 
			} 

			// Write the log file.
			$file  = $upload_dir . '/' . $file . '.log';
			$file  = fopen( $file, $mode );
			$bytes = fwrite( $file, current_time( 'mysql' ) . "::" . $entry . "\n" ); 
			fclose( $file ); 

			return $bytes;
		}
	}
	
}

$glassboxx = new Woo_Glassboxx();
